@@ -1,12 +1,19 @@
-import time, datetime
-from socket import timeout
+import logging
+import socket
 import sys
+import time, datetime
+import types
+import urllib2
 from decorator import decorator
 
-from dogapi.common import SharedCounter
-
+from dogapi.common import SharedCounter, TimeoutManager
 from v1 import *
 
+log = logging.getLogger('dogapi')
+
+import ssl
+timeout_exceptions = (socket.timeout, ssl.SSLError)
+
 class SimpleClient(object):
     """
     A high-level client for interacting with the Datadog API.
@@ -22,10 +29,48 @@ class SimpleClient(object):
     def __init__(self):
         self.api_key = None
         self.application_key = None
-        self.max_timeouts = 3
-        self.timeout_counter = SharedCounter()
         self.timeout = 2
         self.swallow = True
+        self.timeout_manager = TimeoutManager()
+        # FIXME: preserved for backwards compatability. remove when no longer
+        # accessed in templeton
+        self.max_timeouts = 3
+        self.timeout_counter = SharedCounter()
+        self._use_ec2_instance_id = False
+        self._default_host = socket.gethostname()
+    
+    def use_ec2_instance_id():
+        def fget(self):
+            return self._use_ec2_instance_id
+        
+        def fset(self, value):
+            self._use_ec2_instance_id = value
+
+            if value:
+                try:
+                    # Remember the previous default timeout
+                    old_timeout = socket.getdefaulttimeout()
+
+                    # Try to query the EC2 internal metadata service, but fail fast
+                    socket.setdefaulttimeout(0.25)
+
+                    try:
+                        host = urllib2.urlopen(urllib2.Request('http://169.254.169.254/latest/meta-data/instance-id')).read()
+                    finally:
+                        # Reset the previous default timeout
+                        socket.setdefaulttimeout(old_timeout)
+                except Exception:
+                    host = socket.gethostname()
+
+                self._default_host = host
+            else:
+                self._default_host = socket.gethostname()
+        
+        def fdel(self):
+            del self._use_ec2_instance_id
+        
+        return locals()
+    use_ec2_instance_id = property(**use_ec2_instance_id())
 
     @decorator
     def _swallow_exceptions(f, self, *args, **kwargs):
@@ -39,7 +84,7 @@ class SimpleClient(object):
 
     def _report_error(self, message):
         if self.swallow:
-            print >> sys.stderr, message
+            log.error(message)
         else:
             raise Exception(message)
 
@@ -57,7 +102,9 @@ class SimpleClient(object):
         :param value: data point value
         :type value: numeric
 
-        :param host: optional host to scope the metric (e.g. ``"hostA.example.com"``)
+        :param host: optional host to scope the metric (e.g.
+        ``"hostA.example.com"``). defaults to local hostname. to submit without
+        a host, explicitly set host=None.
         :type host: string
 
         :param device: optional device to scope the metric (e.g. ``"eth0"``)
@@ -65,13 +112,29 @@ class SimpleClient(object):
 
         :raises: Exception on failure
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None:
             self._report_error("Metric API requires an api key")
-        s = MetricService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
+            return
+
+        # FIXME import typecheck more elegant than this
+        # Check type of value since .metric and .metrics can easily by confused
+        if type(value) not in (types.FloatType, types.IntType):
+            self._report_error(".metric takes a scalar value not a %s. You might want to use .metrics instead" % type(value))
+            return
+        
+        if host is None:
+            host = self._default_host
+        s = MetricService(self.api_key, self.application_key, timeout=self.timeout)
         now = time.mktime(datetime.datetime.now().timetuple())
-        r = s.post(name, [[now, value]], host=host, device=device)
+        try:
+            r = s.post(name, [[now, value]], host=host, device=device)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
 
@@ -86,7 +149,9 @@ class SimpleClient(object):
         :param values: data series. list of (POSIX timestamp, intever value) tuples. (e.g. ``[(1317652676, 15), (1317652706, 18), ...]``)
         :type values: list
 
-        :param host: optional host to scope the metric (e.g. ``"hostA.example.com"``)
+        :param host: optional host to scope the metric (e.g.
+        ``"hostA.example.com"``). defaults to local hostname. to submit without
+        a host, explicitly set host=None.
         :type host: string
 
         :param device: optional device to scope the metric (e.g. ``"eth0"``)
@@ -94,27 +159,44 @@ class SimpleClient(object):
 
         :raises: Exception on failure
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None:
             self._report_error("Metric API requires an api key")
-        s = MetricService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        if device:
-            r = s.post(name, values, host=host, device=device)
-        else:
-            r = s.post(name, values, host=host)
+            return
+
+        # FIXME import typecheck more elegant than this
+        if type(values) not in (types.ListType, types.TupleType):
+            self._report_error(".metrics takes a list of pairs not a %s. You might want to use .metric instead to send a scalar value" % type(values))
+            return
+
+        if host is None:
+            host = self._default_host
+        s = MetricService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            if device:
+                r = s.post(name, values, host=host, device=device)
+            else:
+                r = s.post(name, values, host=host)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
-            
+
     @_swallow_exceptions
     def batch_metrics(self, values, host=None, device=None):
         """
         Submit a series of metrics with 1 or more data points to the metric API
-        
+
         :param values A dictionary of names to a list values, in the form of {name: [(POSIX timestamp, integer value), ...], name2: [(POSIX timestamp, integer value), ...]}
         :type values: dict
-        
-        :param host: optional host to scope the metric (e.g. ``"hostA.example.com"``)
+
+        :param host: optional host to scope the metric (e.g.
+        ``"hostA.example.com"``). to submit without a host, explicitly set
+        host=None.
         :type host: string
 
         :param device: optional device to scope the metric (e.g. ``"eth0"``)
@@ -122,15 +204,25 @@ class SimpleClient(object):
 
         :raises: Exception on failure
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None:
             self._report_error("Metric API requires an api key")
-        s = MetricService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        if device:
-            r = s.post_batch(values, host=host, device=device)
-        else:
-            r = s.post_batch(values, host=host)
+            return
+
+        if host is None:
+            host = self._default_host
+        s = MetricService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            if device:
+                r = s.post_batch(values, host=host, device=device)
+            else:
+                r = s.post_batch(values, host=host)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
 
     #
     # Comment API
@@ -157,17 +249,25 @@ class SimpleClient(object):
 
         :raises:  Exception on failure
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Comment API requires api and application keys")
-        s = CommentService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        if comment_id is None:
-            r = s.post(handle, message, related_event_id)
-        else:
-            r = s.edit(comment_id, handle, message, related_event_id)
+            return
+        s = CommentService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            if comment_id is None:
+                r = s.post(handle, message, related_event_id)
+            else:
+                r = s.edit(comment_id, handle, message, related_event_id)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['comment']['id']
 
     @_swallow_exceptions
@@ -180,12 +280,19 @@ class SimpleClient(object):
 
         :raises: Exception on error
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Comment API requires api and application keys")
-        s = CommentService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.delete(comment_id)
+            return
+        s = CommentService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.delete(comment_id)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
 
@@ -200,14 +307,22 @@ class SimpleClient(object):
         :return: [ { 'tag1': [ 'host1', 'host2', ... ] }, ... ]
         :rtype: list
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Tag API requires api and application keys")
-        s = TagService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.get_all()
+            return
+        s = TagService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.get_all()
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['tags']
 
     @_swallow_exceptions
@@ -221,14 +336,22 @@ class SimpleClient(object):
         :return: tags for the host
         :rtype: list
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Tag API requires api and application keys")
-        s = TagService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.get(host_id)
+            return
+        s = TagService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.get(host_id)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['tags']
 
     @_swallow_exceptions
@@ -242,12 +365,19 @@ class SimpleClient(object):
         :param tagN: tag name
         :type tagN: string
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Tag API requires api and application keys")
-        s = TagService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.add(host_id, args)
+            return
+        s = TagService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.add(host_id, args)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
 
@@ -262,12 +392,19 @@ class SimpleClient(object):
         :param tagN: tag name
         :type tagN: string
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Tag API requires api and application keys")
-        s = TagService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.update(host_id, args)
+            return
+        s = TagService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.update(host_id, args)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
 
@@ -279,12 +416,19 @@ class SimpleClient(object):
         :param host_id: id or name of the host
         :type host_id: integer or string
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Tag API requires api and application keys")
-        s = TagService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.detach(host_id)
+            return
+        s = TagService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.detach(host_id)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
 
@@ -316,14 +460,22 @@ class SimpleClient(object):
         :return: list of events (see https://github.com/DataDog/dogapi/wiki/Event for structure)
         :rtype: decoded JSON
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Event API requires api and application keys")
-        s = EventService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.query(start, end, priority, sources, tags)
+            return
+        s = EventService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.query(start, end, priority, sources, tags)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['events']
 
     @_swallow_exceptions
@@ -337,18 +489,26 @@ class SimpleClient(object):
         :return: event details (see https://github.com/DataDog/dogapi/wiki/Event for structure)
         :rtype: decoded JSON
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Event API requires api and application keys")
-        s = EventService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.get(id)
+            return
+        s = EventService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.get(id)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['event']
 
     @_swallow_exceptions
-    def event(self, title, text, date_happened=None, handle=None, priority=None, related_event_id=None, tags=None):
+    def event(self, title, text, date_happened=None, handle=None, priority=None, related_event_id=None, tags=None, host=None, device_name=None, **kwargs):
         """
         Post an event.
 
@@ -373,17 +533,31 @@ class SimpleClient(object):
         :param tags: tags to post the event with
         :type tags: list of strings
 
+        :param host: host to post the event with
+        :type host: list of strings
+
+        :param device_name: device_name to post the event with
+        :type device_name: list of strings
+
         :return: new event id
         :rtype: integer
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None:
             self._report_error("Event API requires api key")
-        s = EventService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.post(title, text, date_happened, handle, priority, related_event_id, tags)
+            return
+        s = EventService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.post(title, text, date_happened, handle, priority, related_event_id, tags, host, device_name, **kwargs)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['event']['id']
 
     #
@@ -400,14 +574,22 @@ class SimpleClient(object):
         :return: dashboard definition (see https://github.com/DataDog/dogapi/wiki/Dashboard for details)
         :rtype: decoded JSON
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Dash API requires api and application keys")
-        s = DashService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.get(dash_id)
+            return
+        s = DashService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.get(dash_id)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['dash']
 
     @_swallow_exceptions
@@ -427,14 +609,22 @@ class SimpleClient(object):
         :return: new dashboard's id
         :rtype: integer
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Dash API requires api and application keys")
-        s = DashService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.create(title, description, graphs)
+            return
+        s = DashService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.create(title, description, graphs)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['dash']['id']
 
     @_swallow_exceptions
@@ -457,14 +647,22 @@ class SimpleClient(object):
         :return: dashboard's id
         :rtype: integer
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Dash API requires api and application keys")
-        s = DashService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.update(dash_id, title, description, graphs)
+            return
+        s = DashService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.update(dash_id, title, description, graphs)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['dash']['id']
 
     @_swallow_exceptions
@@ -475,12 +673,19 @@ class SimpleClient(object):
         :param dash_id: dash to delete
         :type dash_id: integer
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Dash API requires api and application keys")
-        s = DashService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.delete(dash_id)
+            return
+        s = DashService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.delete(dash_id)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
 
@@ -498,12 +703,20 @@ class SimpleClient(object):
         :return: a dictionary maping each queried facet to a list of name strings
         :rtype: dictionary
         """
-        if self.timeout_counter.counter >= self.max_timeouts:
-            return None
+        if not self.timeout_manager.should_submit():
+            self._report_error("Too many timeouts. Won't try again for {1} seconds.".format(*self.timeout_manager.backoff_status()))
+            return
         if self.api_key is None or self.application_key is None:
             self._report_error("Search API requires api and application keys")
-        s = SearchService(self.api_key, self.application_key, timeout=self.timeout, timeout_counter=self.timeout_counter)
-        r = s.query(query)
+            return
+        s = SearchService(self.api_key, self.application_key, timeout=self.timeout)
+        try:
+            r = s.query(query)
+        except timeout_exceptions:
+            self.timeout_manager.report_timeout()
+            self._report_error('Client timed out after %d seconds.' % self.timeout)
+            return
         if r.has_key('errors'):
             self._report_error(r['errors'])
+            return
         return r['results']
